@@ -117,9 +117,31 @@ def validator(x: int) -> str:
             ret = None
         self.assertEqual(ret, i_unicode, "chr returned wrong value")
 
+    @given(x=st.integers())
+    @example(0)
+    @example(-1)
+    @example(100)
+    def test_hex(self, x):
+        # this tests that errors that are caused by assignments are actually triggered at the time of assigning
+        source_code = """
+def validator(x: int) -> str:
+    return hex(x)
+        """
+        ast = compiler.parse(source_code)
+        code = compiler.compile(ast)
+        code = code.compile()
+        f = code.term
+        # UPLC lambdas may only take one argument at a time, so we evaluate by repeatedly applying
+        for d in [uplc.PlutusInteger(x)]:
+            f = uplc.Apply(f, d)
+        ret = uplc_eval(f).value.decode("utf8")
+        self.assertEqual(ret, hex(x), "hex returned wrong value")
+
     @given(xs=st.one_of(st.builds(lambda x: str(x), st.integers()), st.text()))
     @example("")
     @example("10_00")
+    @example("_")
+    @example("_1")
     def test_int_string(self, xs: str):
         # this tests that errors that are caused by assignments are actually triggered at the time of assigning
         source_code = """
@@ -176,6 +198,96 @@ def validator(x: List[int]) -> int:
         ret = uplc_eval(f).value
         self.assertEqual(ret, len(xs), "len returned wrong value")
 
+    @given(xs=st.lists(st.integers()))
+    def test_max(self, xs):
+        # this tests that errors that are caused by assignments are actually triggered at the time of assigning
+        source_code = """
+def validator(x: List[int]) -> int:
+    return max(x)
+        """
+        ast = compiler.parse(source_code)
+        code = compiler.compile(ast)
+        code = code.compile()
+        try:
+            exp = max(xs)
+        except ValueError:
+            exp = None
+        try:
+            f = code.term
+            # UPLC lambdas may only take one argument at a time, so we evaluate by repeatedly applying
+            for d in [uplc.PlutusList([uplc.PlutusInteger(int(x)) for x in xs])]:
+                f = uplc.Apply(f, d)
+            ret = uplc_eval(f).value
+        except:
+            ret = None
+        self.assertEqual(ret, exp, "max returned wrong value")
+
+    @given(xs=st.lists(st.integers()))
+    def test_min(self, xs):
+        # this tests that errors that are caused by assignments are actually triggered at the time of assigning
+        source_code = """
+def validator(x: List[int]) -> int:
+    return min(x)
+        """
+        ast = compiler.parse(source_code)
+        code = compiler.compile(ast)
+        code = code.compile()
+        try:
+            exp = min(xs)
+        except ValueError:
+            exp = None
+        try:
+            f = code.term
+            # UPLC lambdas may only take one argument at a time, so we evaluate by repeatedly applying
+            for d in [uplc.PlutusList([uplc.PlutusInteger(int(x)) for x in xs])]:
+                f = uplc.Apply(f, d)
+            ret = uplc_eval(f).value
+        except:
+            ret = None
+        self.assertEqual(ret, exp, "min returned wrong value")
+
+    @given(x=st.integers(), y=st.integers(min_value=0, max_value=20))
+    def test_pow(self, x: int, y: int):
+        source_code = """
+def validator(x: int, y: int) -> int:
+    return pow(x, y) % 10000000000
+        """
+        ast = compiler.parse(source_code)
+        code = compiler.compile(ast)
+        code = code.compile()
+        f = code.term
+        try:
+            exp = pow(x, y) % 10000000000
+        except ValueError:
+            exp = None
+        try:
+            for d in [uplc.PlutusInteger(x), uplc.PlutusInteger(y)]:
+                f = uplc.Apply(f, d)
+            ret = uplc_eval(f).value
+        except:
+            ret = None
+        self.assertEqual(ret, exp, "pow returned wrong value")
+
+    @given(x=st.integers())
+    @example(0)
+    @example(-1)
+    @example(100)
+    def test_oct(self, x):
+        # this tests that errors that are caused by assignments are actually triggered at the time of assigning
+        source_code = """
+def validator(x: int) -> str:
+    return oct(x)
+        """
+        ast = compiler.parse(source_code)
+        code = compiler.compile(ast)
+        code = code.compile()
+        f = code.term
+        # UPLC lambdas may only take one argument at a time, so we evaluate by repeatedly applying
+        for d in [uplc.PlutusInteger(x)]:
+            f = uplc.Apply(f, d)
+        ret = uplc_eval(f).value.decode("utf8")
+        self.assertEqual(ret, oct(x), "oct returned wrong value")
+
     @given(i=st.integers(max_value=100))
     def test_range(self, i):
         # this tests that errors that are caused by assignments are actually triggered at the time of assigning
